@@ -19,20 +19,11 @@ import re
 import struct
 import sys
 import textwrap
-
-try:
-    import io
-except ImportError:
-    # replacement of io.open() for python < 2.6
-    # we use codecs instead
-    class io(object):
-        @staticmethod
-        def open(fpath, mode='r', encoding=None):
-            return codecs.open(fpath, mode, encoding)
+import io
 
 
 __author__ = 'David Jean Louis <izimobil@gmail.com>'
-__version__ = '1.1.0'
+__version__ = '1.2.0'
 __all__ = ['pofile', 'POFile', 'POEntry', 'mofile', 'MOFile', 'MOEntry',
            'default_encoding', 'escape', 'unescape', 'detect_encoding', ]
 
@@ -43,7 +34,7 @@ default_encoding = 'utf-8'
 # python 2/3 compatibility helpers {{{
 
 
-if sys.version_info[:2] < (3, 0):
+if sys.version_info < (3,):
     PY3 = False
     text_type = unicode
 
@@ -102,8 +93,8 @@ def _is_file(filename_or_contents):
         In the latter case, this function will always return False.
     """
     try:
-        return os.path.exists(filename_or_contents)
-    except (ValueError, UnicodeEncodeError):
+        return os.path.isfile(filename_or_contents)
+    except (TypeError, ValueError, UnicodeEncodeError):
         return False
 # }}}
 # function pofile() {{{
@@ -149,7 +140,8 @@ def mofile(mofile, **kwargs):
     Arguments:
 
     ``mofile``
-        string, full or relative path to the mo file or its content (data).
+        string, full or relative path to the mo file or its content (string
+        or bytes).
 
     ``wrapwidth``
         integer, the wrap width, only useful when the ``-w`` option was passed
@@ -202,9 +194,14 @@ def detect_encoding(file, binary_mode=False):
         return True
 
     if not _is_file(file):
-        match = rxt.search(file)
+        try:
+            match = rxt.search(file)
+        except TypeError:
+            match = rxb.search(file)
         if match:
             enc = match.group(1).strip()
+            if not isinstance(enc, text_type):
+                enc = enc.decode('utf-8')
             if charset_exists(enc):
                 return enc
     else:
@@ -215,17 +212,16 @@ def detect_encoding(file, binary_mode=False):
         else:
             mode = 'r'
             rx = rxt
-        f = open(file, mode)
-        for l in f.readlines():
-            match = rx.search(l)
-            if match:
-                f.close()
-                enc = match.group(1).strip()
-                if not isinstance(enc, text_type):
-                    enc = enc.decode('utf-8')
-                if charset_exists(enc):
-                    return enc
-        f.close()
+        with open(file, mode) as f:
+            for line in f.readlines():
+                match = rx.search(line)
+                if match:
+                    f.close()
+                    enc = match.group(1).strip()
+                    if not isinstance(enc, text_type):
+                        enc = enc.decode('utf-8')
+                    if charset_exists(enc):
+                        return enc
     return default_encoding
 # }}}
 # function escape() {{{
@@ -233,13 +229,16 @@ def detect_encoding(file, binary_mode=False):
 
 def escape(st):
     """
-    Escapes the characters ``\\\\``, ``\\t``, ``\\n``, ``\\r`` and ``"`` in
-    the given string ``st`` and returns it.
+    Escapes the characters ``\\\\``, ``\\t``, ``\\n``, ``\\r``, ``\\v``,
+    ``\\b``, ``\\f`` and ``"`` in the given string ``st`` and returns it.
     """
     return st.replace('\\', r'\\')\
              .replace('\t', r'\t')\
              .replace('\r', r'\r')\
              .replace('\n', r'\n')\
+             .replace('\v', r'\v')\
+             .replace('\b', r'\b')\
+             .replace('\f', r'\f')\
              .replace('\"', r'\"')
 # }}}
 # function unescape() {{{
@@ -247,8 +246,8 @@ def escape(st):
 
 def unescape(st):
     """
-    Unescapes the characters ``\\\\``, ``\\t``, ``\\n``, ``\\r`` and ``"`` in
-    the given string ``st`` and returns it.
+    Unescapes the characters ``\\\\``, ``\\t``, ``\\n``, ``\\r``, ``\\v``,
+    ``\\b``, ``\\f`` and ``"`` in the given string ``st`` and returns it.
     """
     def unescape_repl(m):
         m = m.group(1)
@@ -258,10 +257,16 @@ def unescape(st):
             return '\t'
         if m == 'r':
             return '\r'
+        if m == 'v':
+            return '\v'
+        if m == 'b':
+            return '\b'
+        if m == 'f':
+            return '\f'
         if m == '\\':
             return '\\'
         return m  # handles escaped double quote
-    return re.sub(r'\\(\\|n|t|r|")', unescape_repl, st)
+    return re.sub(r'\\(\\|n|t|r|v|b|f|")', unescape_repl, st)
 # }}}
 # function natural_sort() {{{
 
@@ -421,7 +426,7 @@ class _BaseFile(list):
             e.flags.append('fuzzy')
         return e
 
-    def save(self, fpath=None, repr_method='__unicode__'):
+    def save(self, fpath=None, repr_method='__unicode__', newline=None):
         """
         Saves the po file to ``fpath``.
         If it is an existing file and no ``fpath`` is provided, then the
@@ -434,6 +439,9 @@ class _BaseFile(list):
 
         ``repr_method``
             string, the method to use for output.
+
+        ``newline``
+            string, controls how universal newlines works
         """
         if self.fpath is None and fpath is None:
             raise IOError('You must provide a file path to save() method')
@@ -441,13 +449,19 @@ class _BaseFile(list):
         if fpath is None:
             fpath = self.fpath
         if repr_method == 'to_binary':
-            fhandle = open(fpath, 'wb')
+            with open(fpath, 'wb') as fhandle:
+                fhandle.write(contents)
         else:
-            fhandle = io.open(fpath, 'w', encoding=self.encoding)
-            if not isinstance(contents, text_type):
-                contents = contents.decode(self.encoding)
-        fhandle.write(contents)
-        fhandle.close()
+            with io.open(
+                fpath,
+                'w',
+                encoding=self.encoding,
+                newline=newline
+            ) as fhandle:
+                if not isinstance(contents, text_type):
+                    contents = contents.decode(self.encoding)
+                fhandle.write(contents)
+
         # set the file path if not set
         if self.fpath is None and fpath:
             self.fpath = fpath
@@ -691,7 +705,7 @@ class POFile(_BaseFile):
         """
         Convenience method that returns the list of fuzzy entries.
         """
-        return [e for e in self if e.fuzzy]
+        return [e for e in self if e.fuzzy and not e.obsolete]
 
     def obsolete_entries(self):
         """
@@ -828,7 +842,7 @@ class _BaseEntry(object):
             string, the entry msgid_plural.
 
         ``msgstr_plural``
-            list, the entry msgstr_plural lines.
+            dict, the entry msgstr_plural lines.
 
         ``msgctxt``
             string, the entry context (msgctxt).
@@ -906,7 +920,7 @@ class _BaseEntry(object):
         else:
             escaped_field = escape(field)
             specialchars_count = 0
-            for c in ['\\', '\n', '\r', '\t', '"']:
+            for c in ['\\', '\n', '\r', '\t', '\v', '\b', '\f', '"']:
                 specialchars_count += field.count(c)
             # comparison must take into account fieldname length + one space
             # + 2 quotes (eg. msgid "<string>")
@@ -916,7 +930,7 @@ class _BaseEntry(object):
             real_wrapwidth = wrapwidth - flength + specialchars_count
             if wrapwidth > 0 and len(field) > real_wrapwidth:
                 # Wrap the line but take field name into account
-                lines = [''] + [unescape(item) for item in wrap(
+                lines = [''] + [unescape(item) for item in textwrap.wrap(
                     escaped_field,
                     wrapwidth - 2,  # 2 for quotes ""
                     drop_whitespace=False,
@@ -933,6 +947,12 @@ class _BaseEntry(object):
         for line in lines:
             ret.append('%s"%s"' % (delflag, escape(line)))
         return ret
+
+    @property
+    def msgid_with_context(self):
+        if self.msgctxt:
+            return '%s%s%s' % (self.msgctxt, "\x04", self.msgid)
+        return self.msgid
 # }}}
 # class POEntry {{{
 
@@ -989,13 +1009,13 @@ class POEntry(_BaseEntry):
         if self.obsolete:
             comments = [('tcomment', '# ')]
         else:
-            comments = [('comment', '#. '), ('tcomment', '# ')]
+            comments = [('tcomment', '# '), ('comment', '#. ')]
         for c in comments:
             val = getattr(self, c[0])
             if val:
                 for comment in val.split('\n'):
                     if wrapwidth > 0 and len(comment) + len(c[1]) > wrapwidth:
-                        ret += wrap(
+                        ret += textwrap.wrap(
                             comment,
                             wrapwidth,
                             initial_indent=c[1],
@@ -1019,7 +1039,7 @@ class POEntry(_BaseEntry):
                 # what we want for filenames, so the dirty hack is to
                 # temporally replace hyphens with a char that a file cannot
                 # contain, like "*"
-                ret += [l.replace('*', '-') for l in wrap(
+                ret += [line.replace('*', '-') for line in textwrap.wrap(
                     filestr.replace('-', '*'),
                     wrapwidth,
                     initial_indent='#: ',
@@ -1042,7 +1062,7 @@ class POEntry(_BaseEntry):
             prefix = "#| "
         for f in fields:
             val = getattr(self, f)
-            if val:
+            if val is not None:
                 ret += self._str_field(f, prefix, "", val, wrapwidth)
 
         ret.append(_BaseEntry.__unicode__(self, wrapwidth))
@@ -1062,28 +1082,33 @@ class POEntry(_BaseEntry):
         # Work on a copy to protect original
         occ1 = sorted(self.occurrences[:])
         occ2 = sorted(other.occurrences[:])
-        pos = 0
         if occ1 > occ2:
             return 1
         if occ1 < occ2:
             return -1
         # Compare context
-        msgctxt = self.msgctxt or 0
-        othermsgctxt = other.msgctxt or 0
+        msgctxt = self.msgctxt or '0'
+        othermsgctxt = other.msgctxt or '0'
         if msgctxt > othermsgctxt:
             return 1
         elif msgctxt < othermsgctxt:
             return -1
         # Compare msgid_plural
-        msgid_plural = self.msgid_plural or 0
-        othermsgid_plural = other.msgid_plural or 0
+        msgid_plural = self.msgid_plural or '0'
+        othermsgid_plural = other.msgid_plural or '0'
         if msgid_plural > othermsgid_plural:
             return 1
         elif msgid_plural < othermsgid_plural:
             return -1
         # Compare msgstr_plural
-        msgstr_plural = self.msgstr_plural or 0
-        othermsgstr_plural = other.msgstr_plural or 0
+        if self.msgstr_plural and isinstance(self.msgstr_plural, dict):
+            msgstr_plural = list(self.msgstr_plural.values())
+        else:
+            msgstr_plural = []
+        if other.msgstr_plural and isinstance(other.msgstr_plural, dict):
+            othermsgstr_plural = list(other.msgstr_plural.values())
+        else:
+            othermsgstr_plural = []
         if msgstr_plural > othermsgstr_plural:
             return 1
         elif msgstr_plural < othermsgstr_plural:
@@ -1093,7 +1118,6 @@ class POEntry(_BaseEntry):
             return 1
         elif self.msgid < other.msgid:
             return -1
-        return 0
         # Compare msgstr
         if self.msgstr > other.msgstr:
             return 1
@@ -1164,11 +1188,12 @@ class POEntry(_BaseEntry):
     def fuzzy(self):
         return 'fuzzy' in self.flags
 
-    @property
-    def msgid_with_context(self):
-        if self.msgctxt:
-            return '%s%s%s' % (self.msgctxt, "\x04", self.msgid)
-        return self.msgid
+    @fuzzy.setter
+    def fuzzy(self, value):
+        if value and not self.fuzzy:
+            self.flags.insert(0, 'fuzzy')
+        elif not value and self.fuzzy:
+            self.flags.remove('fuzzy')
 
     def __hash__(self):
         return hash((self.msgid, self.msgstr))
@@ -1215,7 +1240,7 @@ class MOEntry(_BaseEntry):
 
 class _POFileParser(object):
     """
-    A finite state machine to parse efficiently and correctly po
+    A finite state machine to efficiently and correctly parse po
     file format.
     """
 
@@ -1305,153 +1330,158 @@ class _POFileParser(object):
         Run the state machine, parse the file line by line and call process()
         with the current matched symbol.
         """
+        try:
+            keywords = {
+                'msgctxt': 'ct',
+                'msgid': 'mi',
+                'msgstr': 'ms',
+                'msgid_plural': 'mp',
+            }
+            prev_keywords = {
+                'msgid_plural': 'pp',
+                'msgid': 'pm',
+                'msgctxt': 'pc',
+            }
+            tokens = []
+            fpath = '%s ' % self.instance.fpath if self.instance.fpath else ''
+            for line in self.fhandle:
+                self.current_line += 1
+                if self.current_line == 1:
+                    BOM = codecs.BOM_UTF8.decode('utf-8')
+                    if line.startswith(BOM):
+                        line = line[len(BOM):]
+                line = line.strip()
+                if line == '':
+                    continue
 
-        keywords = {
-            'msgctxt': 'ct',
-            'msgid': 'mi',
-            'msgstr': 'ms',
-            'msgid_plural': 'mp',
-        }
-        prev_keywords = {
-            'msgid_plural': 'pp',
-            'msgid': 'pm',
-            'msgctxt': 'pc',
-        }
-        tokens = []
-        fpath = '%s ' % self.instance.fpath if self.instance.fpath else ''
-        for line in self.fhandle:
-            self.current_line += 1
-            line = line.strip()
-            if line == '':
-                continue
+                tokens = line.split(None, 2)
+                nb_tokens = len(tokens)
 
-            tokens = line.split(None, 2)
-            nb_tokens = len(tokens)
+                if tokens[0] == '#~|':
+                    continue
 
-            if tokens[0] == '#~|':
-                continue
+                if tokens[0] == '#~' and nb_tokens > 1:
+                    line = line[3:].strip()
+                    tokens = tokens[1:]
+                    nb_tokens -= 1
+                    self.entry_obsolete = 1
+                else:
+                    self.entry_obsolete = 0
 
-            if tokens[0] == '#~' and nb_tokens > 1:
-                line = line[3:].strip()
-                tokens = tokens[1:]
-                nb_tokens -= 1
-                self.entry_obsolete = 1
-            else:
-                self.entry_obsolete = 0
+                # Take care of keywords like
+                # msgid, msgid_plural, msgctxt & msgstr.
+                if tokens[0] in keywords and nb_tokens > 1:
+                    line = line[len(tokens[0]):].lstrip()
+                    if re.search(r'([^\\]|^)"', line[1:-1]):
+                        raise IOError('Syntax error in po file %s(line %s): '
+                                      'unescaped double quote found' %
+                                      (fpath, self.current_line))
+                    self.current_token = line
+                    self.process(keywords[tokens[0]])
+                    continue
 
-            # Take care of keywords like
-            # msgid, msgid_plural, msgctxt & msgstr.
-            if tokens[0] in keywords and nb_tokens > 1:
-                line = line[len(tokens[0]):].lstrip()
-                if re.search(r'([^\\]|^)"', line[1:-1]):
-                    raise IOError('Syntax error in po file %s(line %s): '
-                                  'unescaped double quote found' %
-                                  (fpath, self.current_line))
                 self.current_token = line
-                self.process(keywords[tokens[0]])
-                continue
 
-            self.current_token = line
+                if tokens[0] == '#:':
+                    if nb_tokens <= 1:
+                        continue
+                    # we are on a occurrences line
+                    self.process('oc')
 
-            if tokens[0] == '#:':
-                if nb_tokens <= 1:
-                    continue
-                # we are on a occurrences line
-                self.process('oc')
+                elif line[:1] == '"':
+                    # we are on a continuation line
+                    if re.search(r'([^\\]|^)"', line[1:-1]):
+                        raise IOError('Syntax error in po file %s(line %s): '
+                                      'unescaped double quote found' %
+                                      (fpath, self.current_line))
+                    self.process('mc')
 
-            elif line[:1] == '"':
-                # we are on a continuation line
-                if re.search(r'([^\\]|^)"', line[1:-1]):
-                    raise IOError('Syntax error in po file %s(line %s): '
-                                  'unescaped double quote found' %
-                                  (fpath, self.current_line))
-                self.process('mc')
+                elif line[:7] == 'msgstr[':
+                    # we are on a msgstr plural
+                    self.process('mx')
 
-            elif line[:7] == 'msgstr[':
-                # we are on a msgstr plural
-                self.process('mx')
+                elif tokens[0] == '#,':
+                    if nb_tokens <= 1:
+                        continue
+                    # we are on a flags line
+                    self.process('fl')
 
-            elif tokens[0] == '#,':
-                if nb_tokens <= 1:
-                    continue
-                # we are on a flags line
-                self.process('fl')
+                elif tokens[0] == '#' or tokens[0].startswith('##'):
+                    if line == '#':
+                        line += ' '
+                    # we are on a translator comment line
+                    self.process('tc')
 
-            elif tokens[0] == '#' or tokens[0].startswith('##'):
-                if line == '#':
-                    line += ' '
-                # we are on a translator comment line
-                self.process('tc')
+                elif tokens[0] == '#.':
+                    if nb_tokens <= 1:
+                        continue
+                    # we are on a generated comment line
+                    self.process('gc')
 
-            elif tokens[0] == '#.':
-                if nb_tokens <= 1:
-                    continue
-                # we are on a generated comment line
-                self.process('gc')
+                elif tokens[0] == '#|':
+                    if nb_tokens <= 1:
+                        raise IOError('Syntax error in po file %s(line %s)' %
+                                      (fpath, self.current_line))
 
-            elif tokens[0] == '#|':
-                if nb_tokens <= 1:
+                    # Remove the marker and any whitespace right after that.
+                    line = line[2:].lstrip()
+                    self.current_token = line
+
+                    if tokens[1].startswith('"'):
+                        # Continuation of previous metadata.
+                        self.process('mc')
+                        continue
+
+                    if nb_tokens == 2:
+                        # Invalid continuation line.
+                        raise IOError('Syntax error in po file %s(line %s): '
+                                      'invalid continuation line' %
+                                      (fpath, self.current_line))
+
+                    # we are on a "previous translation" comment line,
+                    if tokens[1] not in prev_keywords:
+                        # Unknown keyword in previous translation comment.
+                        raise IOError('Syntax error in po file %s(line %s): '
+                                      'unknown keyword %s' %
+                                      (fpath, self.current_line,
+                                       tokens[1]))
+
+                    # Remove the keyword and any whitespace
+                    # between it and the starting quote.
+                    line = line[len(tokens[1]):].lstrip()
+                    self.current_token = line
+                    self.process(prev_keywords[tokens[1]])
+
+                else:
                     raise IOError('Syntax error in po file %s(line %s)' %
                                   (fpath, self.current_line))
 
-                # Remove the marker and any whitespace right after that.
-                line = line[2:].lstrip()
-                self.current_token = line
+            if self.current_entry and len(tokens) > 0 and \
+               not tokens[0].startswith('#'):
+                # since entries are added when another entry is found, we must
+                # add the last entry here (only if there are lines). Trailing
+                # comments are ignored
+                self.instance.append(self.current_entry)
 
-                if tokens[1].startswith('"'):
-                    # Continuation of previous metadata.
-                    self.process('mc')
-                    continue
-
-                if nb_tokens == 2:
-                    # Invalid continuation line.
-                    raise IOError('Syntax error in po file %s(line %s): '
-                                  'invalid continuation line' %
-                                  (fpath, self.current_line))
-
-                # we are on a "previous translation" comment line,
-                if tokens[1] not in prev_keywords:
-                    # Unknown keyword in previous translation comment.
-                    raise IOError('Syntax error in po file %s(line %s): '
-                                  'unknown keyword %s' %
-                                  (fpath, self.current_line,
-                                   tokens[1]))
-
-                # Remove the keyword and any whitespace
-                # between it and the starting quote.
-                line = line[len(tokens[1]):].lstrip()
-                self.current_token = line
-                self.process(prev_keywords[tokens[1]])
-
-            else:
-                raise IOError('Syntax error in po file %s(line %s)' %
-                              (fpath, self.current_line))
-
-        if self.current_entry and len(tokens) > 0 and \
-           not tokens[0].startswith('#'):
-            # since entries are added when another entry is found, we must add
-            # the last entry here (only if there are lines). Trailing comments
-            # are ignored
-            self.instance.append(self.current_entry)
-
-        # before returning the instance, check if there's metadata and if
-        # so extract it in a dict
-        metadataentry = self.instance.find('')
-        if metadataentry:  # metadata found
-            # remove the entry
-            self.instance.remove(metadataentry)
-            self.instance.metadata_is_fuzzy = metadataentry.flags
-            key = None
-            for msg in metadataentry.msgstr.splitlines():
-                try:
-                    key, val = msg.split(':', 1)
-                    self.instance.metadata[key] = val.strip()
-                except (ValueError, KeyError):
-                    if key is not None:
-                        self.instance.metadata[key] += '\n' + msg.strip()
-        # close opened file
-        if not isinstance(self.fhandle, list):  # must be file
-            self.fhandle.close()
+            # before returning the instance, check if there's metadata and if
+            # so extract it in a dict
+            metadataentry = self.instance.find('')
+            if metadataentry:  # metadata found
+                # remove the entry
+                self.instance.remove(metadataentry)
+                self.instance.metadata_is_fuzzy = metadataentry.flags
+                key = None
+                for msg in metadataentry.msgstr.splitlines():
+                    try:
+                        key, val = msg.split(':', 1)
+                        self.instance.metadata[key] = val.strip()
+                    except (ValueError, KeyError):
+                        if key is not None:
+                            self.instance.metadata[key] += '\n' + msg.strip()
+        finally:
+            # close opened file
+            if not isinstance(self.fhandle, list):  # must be file
+                self.fhandle.close()
         return self.instance
 
     def add(self, symbol, states, next_state):
@@ -1491,8 +1521,11 @@ class _POFileParser(object):
             if action():
                 self.current_state = state
         except Exception:
-            raise IOError('Syntax error in po file (line %s)' %
-                          self.current_line)
+            fpath = '%s ' % self.instance.fpath if self.instance.fpath else ''
+            if hasattr(self.fhandle, 'close'):
+                self.fhandle.close()
+            raise IOError('Syntax error in po file %s(line %s)' %
+                          (fpath, self.current_line))
 
     # state handlers
 
@@ -1662,7 +1695,10 @@ class _MOFileParser(object):
             whether to check for duplicate entries when adding entries to the
             file (optional, default: ``False``).
         """
-        self.fhandle = open(mofile, 'rb')
+        if _is_file(mofile):
+            self.fhandle = open(mofile, 'rb')
+        else:
+            self.fhandle = io.BytesIO(mofile)
 
         klass = kwargs.get('klass')
         if klass is None:
@@ -1678,7 +1714,7 @@ class _MOFileParser(object):
         Make sure the file is closed, this prevents warnings on unclosed file
         when running tests with python >= 3.2.
         """
-        if self.fhandle:
+        if self.fhandle and hasattr(self.fhandle, 'close'):
             self.fhandle.close()
 
     def parse(self):
@@ -1783,98 +1819,3 @@ class _MOFileParser(object):
             return tup[0]
         return tup
 # }}}
-# class TextWrapper {{{
-
-
-class TextWrapper(textwrap.TextWrapper):
-    """
-    Subclass of textwrap.TextWrapper that backport the
-    drop_whitespace option.
-    """
-    def __init__(self, *args, **kwargs):
-        drop_whitespace = kwargs.pop('drop_whitespace', True)
-        textwrap.TextWrapper.__init__(self, *args, **kwargs)
-        self.drop_whitespace = drop_whitespace
-
-    def _wrap_chunks(self, chunks):
-        """_wrap_chunks(chunks : [string]) -> [string]
-
-        Wrap a sequence of text chunks and return a list of lines of
-        length 'self.width' or less.  (If 'break_long_words' is false,
-        some lines may be longer than this.)  Chunks correspond roughly
-        to words and the whitespace between them: each chunk is
-        indivisible (modulo 'break_long_words'), but a line break can
-        come between any two chunks.  Chunks should not have internal
-        whitespace; ie. a chunk is either all whitespace or a "word".
-        Whitespace chunks will be removed from the beginning and end of
-        lines, but apart from that whitespace is preserved.
-        """
-        lines = []
-        if self.width <= 0:
-            raise ValueError("invalid width %r (must be > 0)" % self.width)
-
-        # Arrange in reverse order so items can be efficiently popped
-        # from a stack of chucks.
-        chunks.reverse()
-
-        while chunks:
-
-            # Start the list of chunks that will make up the current line.
-            # cur_len is just the length of all the chunks in cur_line.
-            cur_line = []
-            cur_len = 0
-
-            # Figure out which static string will prefix this line.
-            if lines:
-                indent = self.subsequent_indent
-            else:
-                indent = self.initial_indent
-
-            # Maximum width for this line.
-            width = self.width - len(indent)
-
-            # First chunk on line is whitespace -- drop it, unless this
-            # is the very beginning of the text (ie. no lines started yet).
-            if self.drop_whitespace and chunks[-1].strip() == '' and lines:
-                del chunks[-1]
-
-            while chunks:
-                length = len(chunks[-1])
-
-                # Can at least squeeze this chunk onto the current line.
-                if cur_len + length <= width:
-                    cur_line.append(chunks.pop())
-                    cur_len += length
-
-                # Nope, this line is full.
-                else:
-                    break
-
-            # The current line is full, and the next chunk is too big to
-            # fit on *any* line (not just this one).
-            if chunks and len(chunks[-1]) > width:
-                self._handle_long_word(chunks, cur_line, cur_len, width)
-
-            # If the last chunk on this line is all whitespace, drop it.
-            if self.drop_whitespace and cur_line and not cur_line[-1].strip():
-                del cur_line[-1]
-
-            # Convert current line back to a string and store it in list
-            # of all lines (return value).
-            if cur_line:
-                lines.append(indent + ''.join(cur_line))
-
-        return lines
-# }}}
-# function wrap() {{{
-
-
-def wrap(text, width=70, **kwargs):
-    """
-    Wrap a single paragraph of text, returning a list of wrapped lines.
-    """
-    if sys.version_info < (2, 6):
-        return TextWrapper(width=width, **kwargs).wrap(text)
-    return textwrap.wrap(text, width=width, **kwargs)
-
-# }}}
